<?php

/**
 * @since      1.0.0
 * @package    Dp_Cpt_Filterable_Module
 * @subpackage Dp_Cpt_Filterable_Module/includes
 * @author     DiviPlugins <support@diviplugins.com>
 */
class Dp_Dfg_Utils {

	/**
	 * Ajax get post data
	 *
	 * @since    1.0.0
	 */
	public static function ajax_get_posts_data() {
		check_ajax_referer( 'dpdfg_get_posts_data_action' );
		$post       = wp_unslash( $_POST );
		$posts_data = array();
		if ( isset( $post['module_data'] ) ) {
			$visual_builder_request = isset( $post['vb'] ) ? ( 'on' === sanitize_text_field( $post['vb'] ) ) : false;
			/*
			 * Sanitize all module data
			 */
			$sanitized_properties = array();
			foreach ( $post['module_data'] as $name => $value ) {
				switch ( $name ) {
					case 'hover_icon':
					case 'read_more_button_icon':
						if ( function_exists( 'et_sanitize_font_icon' ) ) {
							$sanitized_properties[ $name ] = et_sanitize_font_icon( $value );
						} else {
							$sanitized_properties[ $name ] = $value;
						}
						break;
					case 'no_results':
					case 'conditional_tags':
					case 'query_var':
						$sanitized_properties[ $name ] = $value;
						break;
					default:
						$sanitized_properties[ $name ] = sanitize_text_field( $value );
						break;
				}
			}
			/*
			 * Get posts data
			 */
			if ( isset( $post['page'] ) ) {
				$posts_data = self::get_posts_data( $sanitized_properties, $post['page'], 'ajax' );
			} else {
				$posts_data = self::get_posts_data( $sanitized_properties, 1, 'ajax' );
			}
			/*
			 * Get data needed for the frontend or the visual builder
			 */
			if ( $visual_builder_request ) {
				$posts_data['filter_terms'] = self::get_filter_terms( $sanitized_properties, $posts_data['posts'] );
			} else if ( isset( $posts_data['posts'] ) ) {
				$posts_data['posts'] = self::get_items_html( $sanitized_properties, $posts_data['posts'] );
			}
		} else {
			$posts_data['error'] = __( 'Missing module data.', 'dpdfg-dp-divi-filtergrid' );
		}
		wp_send_json( $posts_data );
	}

	/**
	 * Process post data for the VB and the front end
	 *
	 * @param array $props Module properties
	 * @param int $page Page number
	 * @param string $from
	 *
	 * @return array $posts_data
	 * @since 1.0.0
	 */
	public static function get_posts_data( $props, $page, $from ) {
		/*
		 * Fix incorrect page number
		 */
		$page       = ( intval( $page ) <= 0 ) ? 1 : intval( $page );
		$posts_data = array();
		/*
		 * Image size according layout
		 */
		$wxh_size = $props['thumbnail_size'];
		if ( ( '400x284' ) === $wxh_size ) {
			//For backward compatibility
			$width  = ( 'dp-dfg-layout-fullwidth' === $props['items_layout'] || 'dp-dfg-layout-none' === $props['items_layout'] ) ? 1080 : 400;
			$height = ( 'dp-dfg-layout-fullwidth' === $props['items_layout'] || 'dp-dfg-layout-none' === $props['items_layout'] ) ? 9999 : 284;
		} elseif ( 'dfg_full' === $wxh_size ) {
			$width  = 9999;
			$height = 9999;
		} else {
			$wxh_size = explode( 'x', $wxh_size );
			$width    = $wxh_size[0];
			$height   = $wxh_size[1];
		}
		$height               = (int) apply_filters( 'dpdfg_image_thumb_height', $height, $props );
		$width                = (int) apply_filters( 'dpdfg_image_thumb_width', $width, $props );
		$posts_data['ratio']  = $height / $width;
		$posts_data['width']  = $width;
		$posts_data['height'] = $height;
		/*
		 * Init the query
		 */
		$query_args = self::set_query_arguments( $props, $page, $from );
		$query      = new WP_Query( $query_args );
		if ( $query->have_posts() ) {
			/*
			 * Query loop
			 */
			while ( $query->have_posts() ) {
				$post = array();
				$query->the_post();
				$post['id']    = get_the_ID();
				$post['title'] = get_the_title();
				$custom_url    = '';
				if ( 'on' === $props['custom_url'] ) {
					$custom_url = get_post_meta( $post['id'], $props['custom_url_field_name'], true );
				}
				$post['permalink']   = ( '' !== $custom_url ) ? $custom_url : get_the_permalink();
				$post['permalink']   = apply_filters( 'dpdfg_permalink', $post['permalink'], $props, $post['id'] );
				$post['author']      = get_the_author();
				$post['author_link'] = get_author_posts_url( get_post_field( 'post_author', $post['id'] ) );
				$post['date']        = get_the_date( $props['date_format'] );
				$post['comments']    = sprintf( _nx( '%s Comment', '%s Comments', get_comments_number(), 'number of comments', 'dpdfg-dp-divi-filtergrid' ), number_format_i18n( get_comments_number() ) );
				/*
				 * Ajax url
				 */
				$post['ajax_url'] = esc_url(
					wp_nonce_url(
						add_query_arg( 'dp_action', 'dfg_popup_fetch', $post['permalink'] ),
						'dfg_popup_fetch'
					)
				);
				/*
				 * Thumbnail
				 */
				$post_thumbnail_id = get_post_thumbnail_id( $post['id'] );
				if ( $post_thumbnail_id ) {
					$image_data                 = wp_get_attachment_image_src( $post_thumbnail_id, array(
						$width,
						$height,
					) );
					$post['thumbnail']          = $image_data[0];
					$post['thumbnail_width']    = $image_data[1];
					$post['thumbnail_height']   = $image_data[2];
					$post['thumbnail_ratio']    = ( intval( $image_data[1] ) ) ? $image_data[2] / $image_data[1] : 1;
					$post['thumbnail_original'] = get_the_post_thumbnail_url( $post['id'], 'full' );
				} else {
					$post['thumbnail']          = false;
					$post['thumbnail_width']    = 0;
					$post['thumbnail_height']   = 0;
					$post['thumbnail_ratio']    = 0;
					$post['thumbnail_original'] = false;
				}
				$post['thumbnail']          = apply_filters( 'dpdfg_thumbnail_url', $post['thumbnail'], $props, $width, $height, $post['id'] );
				$post['thumbnail_original'] = apply_filters( 'dpdfg_thumbnail_original_url', $post['thumbnail_original'], $props, $post['id'] );
				/*
				 * Custom lightbox images
				 */
				$custom_field_images = array();
				if ( '' !== $props['gallery_cf_name'] ) {
					$custom_field_images = get_post_meta( $post['id'], $props['gallery_cf_name'] );
				}
				$post['post_images'] = apply_filters( 'dp_dfg_custom_lightbox', $custom_field_images, $props );
				/*
				 * Excerpt
				 */
				$post['content'] = apply_filters( 'dpdfg_custom_content', self::custom_excerpt( $props ), $props );
				/*
				 * Post/Filter Terms
				 */
				$post['terms_classes'] = implode( ' ', get_post_class() ) . ' ';
				$post['terms_links']   = array();
				/*
				 * Terms to show
				 */
				if ( 'on' == $props['show_post_meta'] && 'on' == $props['show_terms'] ) {
					$post['terms'] = array();
					$tax_to_show   = array( 'category' );
					if ( '' !== $props['show_terms_taxonomy'] ) {
						$tax_to_show = self::process_comma_separate_list( $props['show_terms_taxonomy'] );
					}
					foreach ( $tax_to_show as $tax ) {
						$the_terms = get_the_terms( $post['id'], $tax );
						if ( ! is_wp_error( $the_terms ) && $the_terms ) {
							$post['terms'] = array_merge( $post['terms'], $the_terms );
						}
					}
					foreach ( $post['terms'] as $term ) {
						$post['terms_links'][] = array(
							'name' => $term->name,
							'link' => get_term_link( $term->term_id ),
						);
					}
				}
				/*
				 * Terms to filter (terms classes for jQuery filtering)
				 */
				if ( 'on' === $props['show_filters'] ) {
					$post['filter_terms'] = array();
					$tax_to_show          = array( 'category' );
					if ( 'on' === $props['multilevel'] && '' !== $props['multilevel_tax_data'] ) {
						$tax_to_show = self::get_multilevel_tax_data( $props );
					} else {
						if ( '' !== $props['filter_taxonomies'] ) {
							$tax_to_show = self::process_comma_separate_list( $props['filter_taxonomies'] );
						}
					}
					foreach ( $tax_to_show as $tax ) {
						$the_terms = get_the_terms( $post['id'], $tax );
						if ( ! is_wp_error( $the_terms ) && $the_terms ) {
							$post['filter_terms'] = array_merge( $post['filter_terms'], $the_terms );
						}
					}
					foreach ( $post['filter_terms'] as $term ) {
						$post['terms_classes'] .= 'dp-dfg-term-id-' . $term->term_id . ' ';
					}
				}
				/*
				 * Custom Content
				 */
				$post['after_read_more'] = apply_filters( 'dpdfg_after_read_more', '', $props );
				/*
				 * Custom fields
				 */
				$post['custom_fields'] = array();
				if ( ! empty( $props['custom_fields'] ) ) {
					$clean         = html_entity_decode( stripslashes( $props['custom_fields'] ) );
					$custom_fields = json_decode( $clean, true );
					if ( is_array( $custom_fields ) && ! empty( $custom_fields ) ) {
						foreach ( $custom_fields as $cf ) {
							$cf_data          = array();
							$cf_data['label'] = $cf['label'];
							if ( class_exists( 'ACF' ) ) {
								$cf_data['value'] = get_field( $cf['name'], $post['id'], true );
							} else {
								$cf_data['value'] = get_post_meta( $post['id'], $cf['name'], true );
							}
							if ( ! empty( $cf_data['value'] ) && $cf_data['value'] ) {
								$post['custom_fields'][] = apply_filters( 'dpdfg_custom_field_output', $cf_data, $cf['name'], $props );
							}
						}
					}
				}
				/*
				 * Include post data
				 */
				$posts_data['posts'][] = $post;
			}
			/*
			 * Found posts
			 */
			$posts_data['found_posts'] = $query->found_posts;
			if ( 'custom' === $props['custom_query'] ) {
				$posts_data['post_number'] = $query->query_vars['posts_per_page'];
			} else {
				$posts_data['post_number'] = intval( $props['post_number'] );
			}
			if ( intval( $props['offset_number'] ) ) {
				$max_pages = ( $posts_data['found_posts'] - intval( $props['offset_number'] ) ) / $posts_data['post_number'];
			} else {
				$max_pages = ( $posts_data['found_posts'] / $posts_data['post_number'] );
			}
			$posts_data['max_pages'] = ceil( $max_pages );
		} else {
			$posts_data['no_results'] = sprintf( '<div class="dp-dfg-no-results">%1$s</div>', et_core_intentionally_unescaped( $props['no_results'], 'html' ) );
		}
		wp_reset_postdata();

		return $posts_data;
	}

	/**
	 * @param $props
	 *
	 * @return array|string[]
	 */
	public static function get_multilevel_tax_data( $props ) {
		$taxonomies          = array();
		$multilevel_tax_data = self::clean_multilevel_tax_data( $props );
		if ( is_array( $multilevel_tax_data ) && ! empty( $multilevel_tax_data ) ) {
			foreach ( $multilevel_tax_data as $tax ) {
				$taxonomies[] = $tax['name'];
			}
		} else {
			$taxonomies = array( 'category' );
		}

		return $taxonomies;
	}

	/**
	 * @param $props
	 *
	 * @return mixed
	 */
	public static function clean_multilevel_tax_data( $props ) {
		$clean = html_entity_decode( stripslashes( $props['multilevel_tax_data'] ) );

		return json_decode( $clean, true );
	}

	/**
	 * Get the taxonomy filter terms
	 *
	 * @param $props
	 * @param $posts_data
	 *
	 * @return array|mixed|void
	 */
	public static function get_filter_terms( $props, $posts_data ) {
		/*
		* Define taxonomies and filters terms based on module selection
		*/
		$filter_terms        = array();
		$terms               = array();
		$multilevel_tax_data = array();
		if ( 'basic' !== $props['custom_query'] ) {
			if ( 'on' === $props['multilevel'] ) {
				if ( '' === $props['multilevel_tax_data'] ) {
					$filter_taxonomies = array( 'category' );
				} else {
					$multilevel_tax_data = self::clean_multilevel_tax_data( $props );
					$filter_taxonomies   = self::get_multilevel_tax_data( $props );
				}
				if ( '' !== $props['filter_terms'] ) {
					$filter_terms = self::process_comma_separate_list( $props['filter_terms'] );
				}
			} else {
				$filter_taxonomies = self::process_comma_separate_list( $props['filter_taxonomies'] );
				if ( '' !== $props['filter_terms'] ) {
					$filter_terms = self::process_comma_separate_list( $props['filter_terms'] );
				}
			}
		} else {
			$filter_taxonomies = array( 'category' );
			if ( ! empty( $props['include_categories'] ) ) {
				$filter_terms = self::process_comma_separate_list( $props['include_categories'] );
			}
		}
		/*
		 * Define the terms to use for filter
		 */
		if ( is_array( $filter_taxonomies ) && ! empty( $filter_taxonomies ) ) {
			$terms_args['taxonomy'] = $filter_taxonomies;
			if ( 'custom' !== $props['filters_sort'] ) {
				$terms_args['orderby'] = $props['filters_sort'];
				if ( 'term_group' !== $props['filters_sort'] ) {
					/*
					 * Ignore order define by plugin Category Order and Taxonomy Terms Order
					 * https://wordpress.org/plugins/taxonomy-terms-order/
					 */
					add_filter( 'to/get_terms_orderby/ignore', function () {
						return true;
					} );
				}
			}
			$terms_args['order'] = $props['filters_order'];
			if ( is_array( $filter_terms ) && ! empty( $filter_terms ) ) {
				$terms_args['include'] = $filter_terms;
			}
			$terms = get_terms( $terms_args );
		}
		/*
		 * Custom order filters terms
		 */
		if ( 'custom' === $props['filters_sort'] && ! empty( $props['filters_custom'] ) ) {
			$terms_names          = self::process_comma_separate_list( $props['filters_custom'] );
			$terms_custom_ordered = array();
			foreach ( $terms_names as $term_name ) {
				foreach ( $terms as $term ) {
					if ( $term->name === $term_name ) {
						$terms_custom_ordered[] = $term;
					}
				}
			}
			if ( ! empty( $terms_custom_ordered ) ) {
				$terms = $terms_custom_ordered;
			}
		}
		/*
		* Define filter levels
		*/
		$filter_levels = array();
		if ( 'off' === $props['multilevel'] && 'off' === $props['hide_all'] ) {
			$filter_levels['all'][] = array(
				'id'   => 'all',
				'name' => ( isset( $props['all_text'] ) && 'All' !== $props['all_text'] ) ? $props['all_text'] : __( 'All', 'dpdfg-dp-divi-filtergrid' ),
			);
		}
		foreach ( $terms as $term ) {
			$tax = 'all';
			if ( 'on' === $props['multilevel'] ) {
				$tax = $term->taxonomy;
				if ( ! isset( $filter_levels[ $tax ] ) ) {
					$filter_levels[ $tax ] = array();
					if ( 'off' === $props['hide_all'] ) {
						$all_text = '';
						foreach ( $multilevel_tax_data as $tax_data ) {
							if ( $tax_data['name'] === $tax && isset( $tax_data['all'] ) ) {
								$all_text = $tax_data['all'];
							}
						}
						$filter_levels[ $tax ][] = array(
							'id'   => 'all',
							'name' => ( '' === $all_text ) ? ( $props['all_text'] . ' ' . get_taxonomy( $tax )->labels->name ) : $all_text
						);
					}
				}
				$filter_levels[ $tax ][] = array(
					'id'   => $term->term_id,
					'name' => $term->name,
				);
			} else {
				$filter_levels[ $tax ][] = array(
					'id'   => $term->term_id,
					'name' => $term->name,
				);
			}
		}
		/*
		 * Order taxonomies
		 */
		if ( 'on' === $props['multilevel'] && ! empty( $multilevel_tax_data ) ) {
			$levels_order = array();
			foreach ( $multilevel_tax_data as $tax ) {
				if ( isset( $filter_levels[ $tax['name'] ] ) ) {
					$levels_order[] = $filter_levels[ $tax['name'] ];
				}
			}
			$filter_levels = $levels_order;
		}
		/*
		 * Custom filters filter
		 */
		if ( 'on' === $props['use_custom_terms_filters'] ) {
			$filter_levels = apply_filters( 'dpdfg_custom_filters', $filter_levels, $props );
		}

		return array(
			'levels'      => $filter_levels,
			'levels_data' => $multilevel_tax_data
		);
	}

	/**
	 * Set the query arguments
	 *
	 * @param array $props Module properties
	 * @param int $page Page number
	 *
	 * @return array $query_args Query arguments
	 * @since 1.0.0
	 */
	public static function set_query_arguments( $props, $page, $from ) {
		$conditional_tags = isset( $props['conditional_tags'] ) ? $props['conditional_tags'] : array(
			'is_user_logged_in' => 'on',
			'is_front_page'     => 'off',
			'is_singular'       => 'on',
			'is_archive'        => 'off',
			'is_search'         => 'off',
			'is_tax'            => 'off',
			'is_author'         => 'off',
			'is_date'           => 'off',
			'is_post_type'      => 'off'
		);
		$query_vars       = isset( $props['query_var'] ) ? $props['query_var'] : 0;
		$the_id           = isset( $props['the_ID'] ) ? $props['the_ID'] : array();
		/*
		 * Defaults arguments
		 */
		$query_args = array(
			'post_type'      => array( 'post' ),
			'post_status'    => 'publish',
			'posts_per_page' => 12,
		);
		/*
		 * Set arguments according query types
		 */
		switch ( $props['custom_query'] ) {
			case 'custom':
				$query_args = apply_filters( 'dpdfg_custom_query_args', $query_args, $props );
				if ( 'on' === $props['show_filters'] && isset( $props['active_filter'] ) && ! empty( $props['active_filter'] ) && ! empty( $props['filter_taxonomies'] ) ) {
					$tax_query_with_filters[] = self::get_filter_tax_query_arguments( $props );
					if ( isset( $query_args['tax_query'] ) ) {
						$tax_query_with_filters['relation'] = 'AND';
						$tax_query_with_filters[]           = array( $query_args['tax_query'] );
					}
					$query_args['tax_query'] = $tax_query_with_filters;
				}
				break;
			case 'advanced':
				if ( isset( $props['multiple_cpt'] ) && '' !== $props['multiple_cpt'] ) {
					$query_args['post_type'] = self::process_comma_separate_list( $props['multiple_cpt'] );
				}
				if ( 'on' === $props['use_taxonomy_terms'] ) {
					$include_taxonomies_array = array();
					if ( isset( $props['multiple_taxonomies'] ) && '' !== $props['multiple_taxonomies'] ) {
						$taxonomies = self::process_comma_separate_list( $props['multiple_taxonomies'] );
						foreach ( $taxonomies as $tax ) {
							if ( taxonomy_exists( $tax ) ) {
								$include_taxonomies_array[] = $tax;
							}
						}
					}
					if ( empty( $include_taxonomies_array ) ) {
						$include_taxonomies_array = array( 'category' );
					}
					$exclude_taxonomies_array = array();
					if ( isset( $props['exclude_taxonomies'] ) && '' !== $props['exclude_taxonomies'] ) {
						$taxonomies = self::process_comma_separate_list( $props['exclude_taxonomies'] );
						foreach ( $taxonomies as $tax ) {
							if ( taxonomy_exists( $tax ) ) {
								$exclude_taxonomies_array[] = $tax;
							}
						}
					}
					if ( empty( $exclude_taxonomies_array ) ) {
						$exclude_taxonomies_array = array( 'post_tag' );
					}
					$query_args['tax_query'] = self::process_tax_query( $props, $include_taxonomies_array, $exclude_taxonomies_array );
				} else {
					$all_tax = array();
					foreach ( $query_args['post_type'] as $pt ) {
						$all_tax = array_merge( $all_tax, get_object_taxonomies( $pt, 'names' ) );
					}
				}
				break;
			case 'basic':
				if ( ! empty( $props['include_categories'] ) ) {
					$query_args['cat'] = $props['include_categories'];
				}
				if ( 'on' === $props['show_filters'] && isset( $props['active_filter'] ) && ! empty( $props['active_filter'] ) ) {
					$query_args['tax_query'][] = self::get_filter_tax_query_arguments( $props );
				}
				break;
			case 'archive':
				if ( isset( $props['multiple_cpt'] ) && '' !== $props['multiple_cpt'] ) {
					$query_args['post_type'] = self::process_comma_separate_list( $props['multiple_cpt'] );
				}
				$query_args              = apply_filters( 'dpdfg_archive_query_args', $query_args, $props );
				$query_args['tax_query'] = array();
				if ( 'on' === $conditional_tags['is_archive'] ) {
					if ( 'on' === $conditional_tags['is_author'] ) {
						$query_args['author'] = $the_id;
					}
					if ( 'on' === $conditional_tags['is_date'] ) {
						$query_args['year']     = $query_vars['year'];
						$query_args['monthnum'] = $query_vars['monthnum'];
						$query_args['day']      = $query_vars['day'];
					}
					if ( 'on' === $conditional_tags['is_tax'] ) {
						$taxonomy                = get_term( $the_id )->taxonomy;
						$query_args['post_type'] = get_taxonomy( $taxonomy )->object_type;
						$query_args['tax_query'] = array(
							array(
								'taxonomy' => $taxonomy,
								'field'    => 'term_id',
								'terms'    => array( $the_id )
							)
						);
					}
					if ( 'on' === $conditional_tags['is_post_type'] ) {
						$query_args['post_type'] = $query_vars['post_type'];
					}
				} else if ( 'on' === $conditional_tags['is_search'] ) {
					$query_args['s'] = $query_vars['s'];
				}
				if ( 'on' === $props['show_filters'] && isset( $props['active_filter'] ) && ! empty( $props['active_filter'] ) ) {
					$tax_query_with_filters[] = self::get_filter_tax_query_arguments( $props );
					if ( isset( $query_args['tax_query'] ) ) {
						$tax_query_with_filters['relation'] = 'AND';
					}
					$query_args['tax_query'] = array_merge( $query_args['tax_query'], $tax_query_with_filters );
				}
				break;
		}
		/*
		 * Common arguments for all query types except custom.
		 */
		if ( 'custom' !== $props['custom_query'] ) {
			$query_args['posts_per_page'] = intval( $props['post_number'] );
			if ( intval( $props['offset_number'] ) ) {
				$query_args['offset'] = ( ( $page - 1 ) * $props['post_number'] ) + intval( $props['offset_number'] );
			}
			$query_args['order']   = $props['order'];
			$query_args['orderby'] = $props['orderby'];
			if ( 'meta_value' === $props['orderby'] && '' !== $props['meta_key'] ) {
				$query_args['meta_key']  = $props['meta_key'];
				$query_args['meta_type'] = $props['meta_type'];
			}
		}
		/*
		 * Pagination
		 */
		$query_args['paged'] = $page;
		/*
		 * Sticky posts, remove current post, post status for logged users
		 */
		if ( 'on' === $props['show_private'] && 'on' === $conditional_tags['is_user_logged_in'] ) {
			$query_args['post_status'] = array( 'publish', 'private' );
		}
		if ( 'on' === $props['sticky_posts'] ) {
			$query_args['ignore_sticky_posts'] = 1;
		}
		if ( 'on' === $props['remove_current_post'] && 'on' === $conditional_tags['is_singular'] ) {
			$query_args['post__not_in'] = array( $the_id );
		}
		/*
		 * Search for Ajax
		 */
		if ( 'on' === $props['show_search'] && 'ajax' === $from && '' !== $query_vars['s'] ) {
			$query_args['s'] = $query_vars['s'];
		}

		return $query_args;
	}

	/**
	 * Get tax query array arguments for the active filter
	 *
	 * @param array $props Module properties
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function get_filter_tax_query_arguments( $props ) {
		$filters_query = array();
		if ( ( isset( $props['multifilter'] ) && 'on' === $props['multifilter'] ) || ( isset( $props['multilevel'] ) && 'on' === $props['multilevel'] ) ) {
			$terms      = explode( '|', $props['active_filter'] );
			$taxonomies = array();
			foreach ( $terms as $term ) {
				$tax = get_term( $term )->taxonomy;
				if ( ! isset( $taxonomies[ $tax ] ) ) {
					$taxonomies[ $tax ] = array();
				}
				$taxonomies[ $tax ][] = $term;
			}
			foreach ( $taxonomies as $tax_name => $terms_array ) {
				$filters_query[] = array(
					'taxonomy'         => $tax_name,
					'terms'            => $terms_array,
					'field'            => 'term_id',
					'operator'         => ( $props['multifilter_relation'] === 'AND' ) ? 'AND' : 'IN',
					'include_children' => false
				);
			}
			if ( ! empty( $taxonomies ) ) {
				if ( count( $taxonomies ) > 1 ) {
					$filters_query['relation'] = ( isset( $props['multilevel_relation'] ) && 'on' === $props['multilevel'] ) ? $props['multilevel_relation'] : $props['multifilter_relation'];
				} else {
					$filters_query = $filters_query[0];
				}
			}
		} else {
			$active_filter_taxonomy = get_term( $props['active_filter'] )->taxonomy;
			$filters_query          = array(
				'taxonomy' => $active_filter_taxonomy,
				'terms'    => $props['active_filter'],
				'field'    => 'term_id',
				'operator' => 'IN',
			);
		}

		return $filters_query;
	}

	/**
	 * Process comma separate list to obtain an array.
	 *
	 * @param string $list List options
	 *
	 * @return array List options
	 */
	public static function process_comma_separate_list( $list ) {
		if ( is_string( $list ) ) {
			$array = explode( ',', $list );
			if ( is_array( $array ) ) {
				foreach ( $array as $key => $value ) {
					$array[ $key ] = trim( $value );
				}
			}

			return $array;
		} else {
			return array();
		}
	}

	/**
	 * Build the tax_query part of the query arguments
	 *
	 * @param array $props Module properties
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function process_tax_query( $props, $include_taxonomies_array, $exclude_taxonomies_array ) {
		$tax_query              = array();
		$tax_query_with_filters = array();
		if ( ! empty( $props['include_terms'] ) ) {
			$terms_ids_array = self::process_comma_separate_list( $props['include_terms'] );
			if ( ! empty( $terms_ids_array ) && is_array( $terms_ids_array ) ) {
				$include_tax_query = array();
				foreach ( $include_taxonomies_array as $taxonomy_name ) {
					$terms_of_taxonomy = array();
					foreach ( $terms_ids_array as $term_id ) {
						if ( get_term( $term_id )->taxonomy === $taxonomy_name ) {
							$terms_of_taxonomy[] = $term_id;
						}
					}
					if ( ! empty( $terms_of_taxonomy ) ) {
						$include_tax_query[] = array(
							'taxonomy'         => $taxonomy_name,
							'terms'            => $terms_of_taxonomy,
							'field'            => 'term_id',
							'operator'         => $props['terms_relation'],
							'include_children' => 'on' === $props['include_children_terms'] ? true : false,
						);
					}
				}
				if ( is_array( $include_tax_query ) && count( $include_tax_query ) > 1 ) {
					$include_tax_query['relation'] = $props['taxonomies_relation'];
				}
			}
		}
		if ( ! empty( $props['exclude_terms'] ) ) {
			$terms_ids_array = self::process_comma_separate_list( $props['exclude_terms'] );
			if ( ! empty( $terms_ids_array ) && is_array( $terms_ids_array ) ) {
				$exclude_tax_query = array();
				foreach ( $exclude_taxonomies_array as $taxonomy_name ) {
					$terms_of_taxonomy = array();
					foreach ( $terms_ids_array as $term_id ) {
						if ( get_term( $term_id )->taxonomy === $taxonomy_name ) {
							$terms_of_taxonomy[] = $term_id;
						}
					}
					if ( ! empty( $terms_of_taxonomy ) ) {
						$exclude_tax_query[] = array(
							'taxonomy' => $taxonomy_name,
							'terms'    => $terms_of_taxonomy,
							'field'    => 'term_id',
							'operator' => 'NOT IN',
						);
					}
				}
				if ( is_array( $exclude_tax_query ) && count( $exclude_tax_query ) > 1 ) {
					$exclude_tax_query['relation'] = $props['exclude_taxonomies_relation'];
				}
			}
		}
		/*
		 * Arrange taxonomy query for the cases when is use both or one of the include or exclude taxonomies
		 */
		if ( ! empty( $include_tax_query ) && ! empty( $exclude_tax_query ) ) {
			$tax_query[]           = $include_tax_query;
			$tax_query[]           = $exclude_tax_query;
			$tax_query['relation'] = 'AND';
		} elseif ( ! empty( $include_tax_query ) && empty( $exclude_tax_query ) ) {
			$tax_query = $include_tax_query;
		} elseif ( empty( $include_tax_query ) && ! empty( $exclude_tax_query ) ) {
			$tax_query = $exclude_tax_query;
		}
		/*
		 * Add filters to the taxonomy query is set
		 */
		if ( 'on' === $props['show_filters'] && isset( $props['active_filter'] ) && ! empty( $props['active_filter'] ) && ! empty( $props['filter_taxonomies'] ) ) {
			$tax_query_with_filters[] = self::get_filter_tax_query_arguments( $props );
			if ( ! empty( $tax_query ) ) {
				$tax_query_with_filters['relation'] = 'AND';
				$tax_query_with_filters[]           = array( $tax_query );
			}

			return $tax_query_with_filters;
		} else {
			return $tax_query;
		}
	}

	/**
	 * Get excerpt or truncate content if excerpt is not set.
	 *
	 * @param array $props Module props
	 *
	 * @return string excerpt
	 */
	public static function custom_excerpt( $props ) {
		if ( 'off' === $props['show_content'] ) {
			return '';
		} else {
			if ( has_excerpt() ) {
				return get_the_excerpt();
			} else {
				$post_content = wp_strip_all_tags( get_the_content() );
				$post_content = self::et_strip_shortcodes( $post_content );
				if ( isset( $props['truncate_content'] ) && intval( $props['truncate_content'] ) ) {
					return self::truncate_content( $post_content, $props['truncate_content'] );
				} else {
					return $post_content;
				}
			}
		}
	}

	/**
	 * Custom function to strip shortcodes from the content.
	 *
	 * @param string $content
	 *
	 * @return string
	 * @since 1.0.0
	 */
	public static function et_strip_shortcodes( $content ) {
		$content                  = trim( $content );
		$strip_content_shortcodes = array( 'et_pb_code', 'et_pb_fullwidth_code' );
		foreach ( $strip_content_shortcodes as $shortcode_name ) {
			$regex   = sprintf( '(\[%1$s[^\]]*\][^\[]*\[\/%1$s\])', esc_html( $shortcode_name ) );
			$content = preg_replace( $regex, '', $content );
		}
		$content = preg_replace( '(\[[^\]]+\])', '', $content );

		return $content;
	}

	/**
	 * Limit the content to a number of characters.
	 *
	 * @param string $excerpt Post Excerpt
	 * @param int $excerpt_limit Number of character to truncate the content
	 *
	 * @return string
	 */
	public static function truncate_content( $excerpt, $excerpt_limit ) {
		$charlength = $excerpt_limit;
		$charlength ++;
		if ( mb_strlen( $excerpt ) > $charlength ) {
			$subex   = mb_substr( $excerpt, 0, $charlength - 5 );
			$exwords = explode( ' ', $subex );
			$excut   = - ( mb_strlen( $exwords[ count( $exwords ) - 1 ] ) );
			if ( $excut < 0 ) {
				return mb_substr( $subex, 0, $excut ) . '...';
			} else {
				return $subex;
			}
		} else {
			return $excerpt;
		}
	}

	/**
	 * Process html of each item for the ajax request
	 *
	 * @since    1.0.0
	 */
	public static function get_items_html( $props, $posts ) {
		ob_start();
		$data_icon_overlay = sprintf( ' data-icon="%1$s"', esc_attr( et_pb_process_font_icon( $props['hover_icon'] ) ) );
		foreach ( $posts as $post ) {
			/*
			 * Escape all data coming from $posts
			 */
			$id                 = intval( $post['id'] );
			$permalink          = esc_url( $post['permalink'] );
			$date               = esc_html( $post['date'] );
			$classes            = esc_attr( $post['terms_classes'] );
			$title              = esc_attr( $post['title'] );
			$thumbnail          = esc_attr( $post['thumbnail'] );
			$thumbnail_original = esc_attr( $post['thumbnail_original'] );
			$images             = ( count( $post['post_images'] ) > 0 ) ? esc_attr( implode( '||', $post['post_images'] ) ) : '';
			$ajax_url           = esc_url( $post['ajax_url'] );
			$header_level       = esc_html( $props['dpdfg_entry_title_level'] );
			$author_prefix      = esc_attr( $props['author_prefix_text'] );
			$author             = esc_attr( $post['author'] );
			$author_link        = esc_url( $post['author_link'] );
			/*
			 * Post Data
			 */
			$new_tab             = ( 'on' === $props['read_more_window'] ) ? 'target="_blank"' : '';
			$article_init_output = sprintf(
				'<article id="%1$s" class="%2$s" %3$s %4$s %5$s %6$s>',
				'post-' . $id,
				'dp-dfg-item ' . $classes,
				( 'link' === $props['thumbnail_action'] && ( 'off' === $props['show_thumbnail'] || ! $post['thumbnail'] ) ) ? ' data-link="' . $permalink . '"' : '',
				( 'popup' === $props['thumbnail_action'] && ( 'off' === $props['show_thumbnail'] || ! $post['thumbnail'] ) ) ? ' data-ajax-url="' . $ajax_url . '"' : '',
				( ( 'lightbox' === $props['thumbnail_action'] || 'lightbox_gallery' === $props['thumbnail_action'] ) && 'off' === $props['show_thumbnail'] && $post['thumbnail'] ) ? ' data-img="' . $thumbnail_original . '" data-title="' . $title . '"' : '',
				( 'gallery_cf' === $props['thumbnail_action'] && ( 'off' === $props['show_thumbnail'] || ! $post['thumbnail'] ) ? ' data-images="' . $images . '"' : ''
				)
			);
			echo et_core_intentionally_unescaped( $article_init_output, 'html' );
			/*
			 * Thumbnail
			 */
			if ( 'on' === $props['show_thumbnail'] ) {
				$thumbnail_output = '';
				if ( $post['thumbnail'] ) {
					$ratio        = ( intval( $post['thumbnail_width'] ) ) ? $post['thumbnail_height'] / $post['thumbnail_width'] : 1;
					$image_output = sprintf( '<img src="%1$s" alt="%2$s" width="%3$s" height="%4$s" data-ratio="%5$s"/>', $thumbnail, $title, $post['thumbnail_width'], $post['thumbnail_height'], $ratio );
					if ( 'none' === $props['thumbnail_action'] ) {
						$thumbnail_output = $image_output;
					} elseif ( 'link' === $props['thumbnail_action'] ) {
						$thumbnail_output = sprintf( '<a href="%2$s" class="dp-dfg-image-link" %3$s>%1$s</a>', $image_output, $permalink, $new_tab );
					} elseif ( 'popup' === $props['thumbnail_action'] ) {
						$thumbnail_output = sprintf( '<a href="%2$s" data-ajax-url="%3$s" class="dp-dfg-image-link entry-thumb-popup">%1$s</a>', $image_output, $permalink, $ajax_url );
					} elseif ( 'lightbox' === $props['thumbnail_action'] || 'lightbox_gallery' === $props['thumbnail_action'] ) {
						$thumbnail_output = sprintf( '<a href="%3$s" data-img="%4$s" data-title="%2$s" class="dp-dfg-image-link entry-thumb-lightbox">%1$s</a>', $image_output, $title, $permalink, $thumbnail_original );
					} elseif ( 'gallery_cf' === $props['thumbnail_action'] ) {
						if ( count( $post['post_images'] ) > 0 ) {
							$thumbnail_output = sprintf( '<a href="#" data-images="%2$s" class="dp-dfg-image-link entry-thumb-gallery-cf">%1$s</a>', $image_output, $images );
						} else {
							$thumbnail_output = $image_output;
						}
					}
				}
				$custom_thumbnail = apply_filters( 'dpdfg_custom_thumbnail', $thumbnail_output, $props, $id );
				echo et_core_intentionally_unescaped( sprintf( '<figure class="dp-dfg-image entry-thumb">%1$s</figure>', $custom_thumbnail ), 'html' );
			}
			/*
			 * Overlay
			 */
			$overlay        = sprintf( '<span class="et_overlay et_pb_inline_icon" %1$s></span>', $data_icon_overlay );
			$custom_overlay = apply_filters( 'dpdfg_custom_overlay', $overlay, $props, $id );
			if ( 'on' === $props['use_overlay'] ) {
				echo et_core_intentionally_unescaped( sprintf( '<div class="dp-dfg-overlay">%1$s</div>', $custom_overlay ), 'html' );
			} else {
				if ( $props['items_layout'] === 'dp-dfg-layout-list' ) {
					echo et_core_intentionally_unescaped( sprintf( '<div class="dp-dfg-overlay dp-dfg-hide">%1$s</div>', $custom_overlay ), 'html' );
				}
			}
			/*
			 * Title
			 */
			if ( 'on' === $props['show_title'] ) {
				if ( 'on' === $props['title_link'] ) {
					$title_output = sprintf( '<%2$s class="entry-title"><a href="%3$s" %4$s>%1$s</a></%2$s>', $title, $header_level, $permalink, $new_tab );
				} else {
					$title_output = sprintf( '<%2$s class="entry-title">%1$s</%2$s>', $title, $header_level );
				}
				$custom_title = apply_filters( 'dpdfg_custom_header', $title_output, $props, $id );
				echo et_core_intentionally_unescaped( sprintf( '<div class="dp-dfg-header entry-header">%1$s</div>', $custom_title ), 'html' );
			}
			/*
			 * Post Meta
			 */
			if ( 'on' === $props['show_post_meta'] ) {
				$meta_output = '';
				/*
				 * Author
				 */
				if ( 'on' === $props['show_author'] ) {
					$meta_output .= sprintf( '<span class="author vcard"><a href="%3$s">%1$s</a>%2$s</span>', $author_prefix . ' ' . $author, ( 'on' === $props['show_date'] || 'on' === $props['show_terms'] || 'on' === $props['show_comments'] ) ? ' | ' : '', $author_link );
				}
				/*
				 * Date
				 */
				if ( 'on' === $props['show_date'] ) {
					$meta_output .= sprintf( '<span class="published">%1$s%2$s</span>', $date, ( 'on' === $props['show_terms'] || 'on' === $props['show_comments'] ) ? ' | ' : '' );
				}
				/*
				 * Terms
				 */
				if ( 'on' === $props['show_terms'] ) {
					if ( isset( $post['terms_links'] ) && ! empty( $post['terms_links'] ) ) {
						$links = array();
						foreach ( $post['terms_links'] as $term ) {
							$links[] = sprintf( '<a href="%1$s" class="term-link">%2$s</a>', esc_url( $term['link'] ), esc_html( $term['name'] ) );
						}
						$meta_output .= sprintf( '<span class="terms">%1$s%2$s</span>', implode( ', ', $links ), ( 'on' === $props['show_comments'] ) ? ' | ' : '' );
					}
				}
				/*
				 * Comments
				 */
				if ( 'on' === $props['show_comments'] ) {
					$meta_output .= sprintf( '<span class="comments">%1$s</span>', esc_html( $post['comments'] ) );
				}
				$custom_meta = apply_filters( 'dpdfg_custom_meta', $meta_output, $props, $id );
				echo et_core_intentionally_unescaped( sprintf( '<div class="dp-dfg-meta entry-meta">%1$s</div>', $custom_meta ), 'html' );
			}
			/*
			 * Excerpt
			 */
			if ( 'off' !== $props['show_content'] ) {
				echo et_core_intentionally_unescaped( sprintf( '<div class="dp-dfg-content entry-summary">%1$s</div>', $post['content'] ), 'html' );
			}
			/*
			 * Read more
			 */
			if ( '' === $props['read_more_button_icon'] ) {
				$props['read_more_button_icon'] = '%%20%%';
			}
			if ( 'on' === $props['read_more'] && '' !== $props['read_more_text'] ) {
				echo et_core_intentionally_unescaped( sprintf( '<div class="et_pb_button_wrapper"><a class="et_pb_button dp-dfg-more-button" href="%1$s" data-icon="%3$s" %4$s>%2$s</a></div>', $permalink, esc_html( $props['read_more_text'] ), esc_attr( et_pb_process_font_icon( $props['read_more_button_icon'] ) ), 'on' === $props['read_more_window'] ? 'target="_blank"' : '' ), 'html' );
			}
			/*
			 * Custom Content
			 */
			if ( 'on' === $props['show_custom_content'] || 'on' === $props['show_custom_fields'] ) {
				$custom_fields = '';
				if ( 'on' === $props['show_custom_fields'] && ! empty( $post['custom_fields'] ) ) {
					foreach ( $post['custom_fields'] as $cf ) {
						if ( ! empty( $cf['value'] ) ) {
							$custom_fields .= sprintf( '<p class="dp-dfg-custom-field">%1$s<span class="dp-dfg-custom-field-value">%2$s</span></p>', ( ! empty( $cf['label'] ) ) ? '<span class="dp-dfg-custom-field-label">' . esc_html( $cf['label'] ) . '</span>' : '', et_core_intentionally_unescaped( $cf['value'], 'html' ) );
						}
					}
				}
				$custom_content = '';
				if ( 'on' === $props['show_custom_content'] ) {
					$custom_content = $post['after_read_more'];
				}
				if ( 'on' === $props['custom_content_container'] ) {
					echo et_core_intentionally_unescaped( sprintf( '<div class="dp-dfg-custom-content">%1$s</div>', $custom_fields . $custom_content ), 'html' );
				} else {
					if ( '' !== $custom_fields ) {
						echo et_core_intentionally_unescaped( sprintf( '%1$s%2$s', $custom_fields, $custom_content ), 'html' );
					} else {
						echo et_core_intentionally_unescaped( $custom_content, 'html' );
					}
				}
			}
			echo '</article><!-- DPDFG End Post Item Container -->';
		}

		return ob_get_clean();
	}

	/**
	 * Ajax action to return the form to select the multiple post types on the VB
	 *
	 * @return mixed
	 * @since 1.0.0
	 */
	public static function ajax_get_cpt() {
		check_ajax_referer( 'dpdfg_get_cpt_action' );
		$html_output = '<form id="dp-dfg-cpt-form">';
		$html_output .= sprintf( '<p>%1$s</p>', __( 'Select one or more post types below. Use CTRL or SHIFT to select multiple.', 'dpdfg-dp-divi-filtergrid' ) );
		$html_output .= '<select class="dp-dfg-vb-select" name="dp-dfg-vb-select" multiple size="6">';
		foreach ( self::get_cpt() as $key => $cpt ) {
			$html_output .= sprintf( '<option value="%1$s">%2$s</option>', esc_html( $key ), esc_html( $cpt ) );
		}
		$html_output .= '</select>';
		$html_output .= self::vb_modal_actions();
		$html_output .= '</form>';
		echo et_core_intentionally_unescaped( $html_output, 'html' );
		wp_die();
	}

	/**
	 * Get custom post types. Use the filter dpcptfm_default_post_types to modified the default post types.
	 *
	 * @return array $options Array with post types name as key and post type label as value.
	 * @since 1.0.0
	 */
	public static function get_cpt() {
		$options           = array();
		$default_post_type = apply_filters( 'dpdfg_default_post_types', array( 'post' => get_post_type_object( 'post' ) ) );
		$post_types        = array_merge(
			$default_post_type,
			get_post_types(
				array(
					'_builtin' => false,
					'public'   => true,
				),
				'objects'
			)
		);
		foreach ( $post_types as $pt ) {
			$options[ $pt->name ] = $pt->label;
		}

		return $options;
	}

	/**
	 * VB selection modal actions.
	 *
	 * @return string
	 * @since 1.0.0
	 */
	public static function vb_modal_actions() {
		$html_output = '<div class="dp-dfg-vb-actions">';
		$html_output .= sprintf( '<input class="dp-dfg-vb-submit" type="button" value="%1$s" />', __( 'Set Values', 'dpdfg-dp-divi-filtergrid' ) );
		$html_output .= sprintf( '<input class="dp-dfg-vb-clean" type="button" value="%1$s" />', __( 'Clean Values', 'dpdfg-dp-divi-filtergrid' ) );
		$html_output .= sprintf( '<input class="dp-dfg-vb-finish" type="button" value="%1$s" />', __( 'Exit', 'dpdfg-dp-divi-filtergrid' ) );
		$html_output .= '</div>';

		return $html_output;
	}

	/**
	 * Ajax action to return the form to select the multiple taxonomies on the VB
	 *
	 * @since 1.0.0
	 */
	public static function ajax_get_taxonomies() {
		check_ajax_referer( 'dpdfg_get_taxonomies_action' );
		$cpt_array = array( 'post' );
		if ( isset( $_POST['cpt'] ) ) {
			$cpts = sanitize_text_field( wp_unslash( $_POST['cpt'] ) );
			if ( substr_count( $cpts, ',' ) > 0 ) {
				$cpt_array = self::process_comma_separate_list( $cpts );
			} else {
				$cpt_array = array( $cpts );
			}
		}
		$html_output = '<form id="dp-dfg-tax-form">';
		$html_output .= sprintf( '<p>%1$s</p>', __( 'Select one or more taxonomies below. Use CTRL or SHIFT to select multiple.', 'dpdfg-dp-divi-filtergrid' ) );
		$html_output .= '<select class="dp-dfg-vb-select" name="dp-dfg-vb-select" multiple size="6">';
		foreach ( self::get_taxonomies( $cpt_array ) as $key => $tax ) {
			$html_output .= sprintf( '<option value="%1$s">%2$s</option>', esc_html( $key ), esc_html( $tax ) );
		}
		$html_output .= '</select>';
		$html_output .= self::vb_modal_actions();
		$html_output .= '</form>';
		echo et_core_intentionally_unescaped( $html_output, 'html' );
		wp_die();
	}

	/**
	 * Get public taxonomies. Use the filter dpcptfm_blacklisted_taxonomies to remove unwanted taxonomies.
	 *
	 * @param array $cpt Array with post types names
	 *
	 * @return array $options Array with taxonomy name as key and taxonomy label as value.
	 * @since 1.0.0
	 */
	public static function get_taxonomies( $cpt ) {
		$options                = array();
		$blacklisted_taxonomies = apply_filters( 'dpdfg_blacklisted_taxonomies', array(
				'layout_category',
				'layout_pack',
				'layout_type',
				'scope',
				'module_width',
				'post_format',
			)
		);
		$taxonomies             = array_diff(
			get_taxonomies(
				array(
					'public'    => true,
					'query_var' => true,
				)
			),
			$blacklisted_taxonomies
		);
		if ( 'all' === $cpt[0] ) {
			foreach ( $taxonomies as $tax ) {
				$tax_obj         = get_taxonomy( $tax );
				$options[ $tax ] = $tax_obj->label;
			}
		} else {
			foreach ( $taxonomies as $tax ) {
				$tax_obj  = get_taxonomy( $tax );
				$is_there = array_intersect( $cpt, $tax_obj->object_type );
				if ( ! empty( $is_there ) ) {
					$options[ $tax ] = $tax_obj->label;
				}
			}
		}

		return apply_filters( 'dpdfg_allowed_taxonomies', $options );;
	}

	/**
	 * Ajax action to return the form to select the multiple taxonomies on the VB
	 *
	 * @since 1.0.0
	 */
	public static function ajax_get_taxonomies_terms() {
		check_ajax_referer( 'dpdfg_get_taxonomies_terms_action' );
		$tax_array = array( 'category' );
		if ( isset( $_POST['tax'] ) ) {
			$taxs = sanitize_text_field( wp_unslash( $_POST['tax'] ) );
			if ( substr_count( $taxs, ',' ) > 0 ) {
				$tax_array = self::process_comma_separate_list( $taxs );
			} else {
				$tax_array = array( $taxs );
			}
		}
		$cpt_array = array( 'post' );
		if ( isset( $_POST['cpt'] ) ) {
			$cpts = sanitize_text_field( wp_unslash( $_POST['cpt'] ) );
			if ( substr_count( $cpts, ',' ) > 0 ) {
				$cpt_array = self::process_comma_separate_list( $cpts );
			} else {
				$cpt_array = array( $cpts );
			}
		}
		$html_output = '<form id="dp-dfg-terms-form">';
		$html_output .= sprintf( '<p>%1$s</p>', __( 'Select one or more terms below. Use CTRL or SHIFT to select multiple.', 'dpdfg-dp-divi-filtergrid' ) );
		$html_output .= '<select class="dp-dfg-vb-select" name="dp-dfg-vb-select" multiple size="12">';
		foreach ( self::get_taxonomies_terms( $tax_array, $cpt_array ) as $tax => $terms ) {
			$html_output .= '<optgroup label="' . esc_html( $tax ) . '">';
			$html_output .= self::show_taxonomy_hierarchy( '', $terms );
			$html_output .= '</optgroup>';
		}
		$html_output .= '</select>';
		$html_output .= self::vb_modal_actions();
		$html_output .= '</form>';
		echo et_core_intentionally_unescaped( $html_output, 'html' );
		wp_die();
	}

	/**
	 * @param $html_output
	 * @param $terms
	 * @param int $level
	 *
	 * @return string
	 */
	public static function show_taxonomy_hierarchy( $html_output, $terms, $level = 0 ) {
		foreach ( $terms as $term ) {
			$html_output .= sprintf( '<option value="%1$s">%3$s %2$s</option>', esc_html( $term->term_id ), esc_html( $term->name ) . ' (' . $term->count . ')', str_repeat( '-', $level ) );
			if ( ! empty( $term->children ) ) {
				$html_output .= self::show_taxonomy_hierarchy( '', $term->children, ( $level + 1 ) );
			}
		}

		return $html_output;
	}

	/**
	 * @param $taxonomy
	 * @param int $parent
	 *
	 * @return array
	 */
	public static function get_taxonomy_hierarchy( $taxonomy, $parent = 0 ) {
		$terms    = get_terms( $taxonomy, array(
			'parent'     => $parent,
			'hide_empty' => false
		) );
		$children = array();
		foreach ( $terms as $term ) {
			$term->children             = self::get_taxonomy_hierarchy( $taxonomy, $term->term_id );
			$children[ $term->term_id ] = $term;
		}

		return $children;
	}

	/**
	 * Get taxonomy terms
	 *
	 * @param array $tax Array with taxonomy names
	 * @param array $cpt Array with post types names
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function get_taxonomies_terms( $tax, $cpt ) {
		$options            = array();
		$all_cpt_taxonomies = self::get_taxonomies( $cpt );
		foreach ( $all_cpt_taxonomies as $tax_name => $tax_label ) {
			if ( 'all' === $tax[0] || in_array( $tax_name, $tax, true ) ) {
				$terms = self::get_taxonomy_hierarchy( $tax_name );
				if ( ! empty( $terms ) ) {
					$options[ $tax_label . ' (' . $tax_name . ')' ] = $terms;
				}
			}
		}

		return $options;
	}

	/**
	 * Ajax action to return custom fields UI
	 *
	 * @since 1.0.7
	 */
	public static function ajax_get_custom_fields() {
		check_ajax_referer( 'dpdfg_get_custom_fields_action' );
		$html_output = sprintf( '<form id="dp-dfg-cf-form"><p>%1$s</p><div id="dp-dfg-cf-container"><input id="dp-dfg-cf-select" type="text"/><span class="dp-dfg-cf-plus dashicons dashicons-plus-alt"></span></div><div class="dp-dfg-cf-selection"></div>%2$s</form>',
			__( 'Begin typing the custom field you want to add and click the + to add it. Once added, you can optionally add a label before each custom field by typing in the input for that field. Leave input empty to display the custom field value without a label. Drag and drop custom fields to change their order.', 'dpdfg-dp-divi-filtergrid' ),
			self::vb_modal_actions()
		);
		echo et_core_intentionally_unescaped( $html_output, 'html' );
		wp_die();
	}

	/**
	 * Get custom fields
	 *
	 * @return array
	 * @since 1.0.7
	 */
	public static function get_custom_fields() {
		$options = array();
		global $wpdb;
		$table_name = $wpdb->base_prefix . 'postmeta';
		$result     = $wpdb->get_results( $wpdb->prepare( 'SELECT DISTINCT meta_key FROM %1s;', $table_name ), ARRAY_A );
		if ( is_array( $result ) ) {
			foreach ( $result as $cf ) {
				$options[] = $cf['meta_key'];
			}
		}

		return $options;
	}

	/**
	 * Ajax action to return multilevel tax UI
	 *
	 * @since 1.4
	 */
	public static function ajax_get_multilevel_tax() {
		check_ajax_referer( 'dpdfg_get_multilevel_tax_action' );
		$options = '';
		foreach ( self::get_taxonomies( array( 'all' ) ) as $tax => $name ) {
			$options .= sprintf( '<option value="%2$s">%1$s</option>', esc_html( $name ), esc_html( $tax ) );
		}
		$html_output = sprintf( '<form id="dp-dfg-cf-form"><p>%1$s</p><div id="dp-dfg-cf-container"><select id="dp-dfg-cf-select">%3$s</select><span class="dp-dfg-cf-plus dashicons dashicons-plus-alt"></span></div><div class="dp-dfg-cf-selection"></div>%2$s</form>',
			__( 'Select the taxonomy you want to add for each level and click the + to add it. Once added, you can optionally add a label before each level by typing in the input for that level. Leave input empty to display levels without a label. Drag and drop levels to change their order.', 'dpdfg-dp-divi-filtergrid' ),
			self::vb_modal_actions(),
			$options
		);
		echo et_core_intentionally_unescaped( $html_output, 'html' );
		wp_die();
	}

	/**
	 * Return an array with the skins options. Use the key for the skin class and the value for the skin name. Use the filter dpdfg_custom_skins to modified the default options.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function get_skins() {
		$defaults = array(
			'dp-dfg-skin-none'                                      => __( 'None', 'dpdfg-dp-divi-filtergrid' ),
			'dp-dfg-skin-default'                                   => __( 'Default', 'dpdfg-dp-divi-filtergrid' ),
			'dp-dfg-skin-default dp-dfg-skin-midnight'              => __( 'Midnight', 'dpdfg-dp-divi-filtergrid' ),
			'dp-dfg-skin-default dp-dfg-skin-left-vertical-filters' => __( 'Left Vertical Filters', 'dpdfg-dp-divi-filtergrid' ),
			'dp-dfg-skin-default dp-dfg-skin-itemsinoverlay'        => __( 'Items in Overlay', 'dpdfg-dp-divi-filtergrid' ),
			'dp-dfg-skin-default dp-dfg-skin-zoomimage'             => __( 'Zoom Image', 'dpdfg-dp-divi-filtergrid' ),
			'dp-dfg-skin-default dp-dfg-skin-library'               => __( 'Library', 'dpdfg-dp-divi-filtergrid' ),
		);

		return array_merge( $defaults, apply_filters( 'dpdfg_custom_skins', $defaults ) );
	}

	/**
	 * Get all registered thumbnail sizes
	 *
	 * @return array
	 * @global array $_wp_additional_image_sizes
	 * @since 1.1.2
	 */
	public static function get_registered_thumbnail_sizes() {
		$options['dfg_full'] = __( 'Original Image Uploaded', 'dpdfg-dp-divi-filtergrid' );
		global $_wp_additional_image_sizes;
		foreach ( get_intermediate_image_sizes() as $_size ) {
			$size_width  = '';
			$size_height = '';
			if ( in_array(
				$_size,
				array(
					'thumbnail',
					'medium',
					'medium_large',
					'large',
				),
				true
			) ) {
				$size_width  = get_option( "{$_size}_size_w" );
				$size_height = get_option( "{$_size}_size_h" );
			} elseif ( isset( $_wp_additional_image_sizes[ $_size ] ) ) {
				$size_width  = $_wp_additional_image_sizes[ $_size ]['width'];
				$size_height = $_wp_additional_image_sizes[ $_size ]['height'];
			}
			$size_key             = $size_width . 'x' . $size_height;
			$size_label           = $_size . ' - (' . $size_width . 'x' . $size_height . ')';
			$options[ $size_key ] = $size_label;
		}

		return $options;
	}

	public static function enqueue_and_localize_cpt_modal_script() {
		wp_enqueue_style( 'dp-dfg-admin-cpt-modal', DPDFG_URL . 'admin/css/dp-dfg-admin.min.css', array(), DPDFG_VERSION, 'all' );
		wp_enqueue_script( 'dp-dfg-admin-cpt-modal', DPDFG_URL . 'admin/js/dp-dfg-admin.min.js', array(
			'jquery',
			'jquery-ui-sortable',
			'jquery-ui-autocomplete'
		), DPDFG_VERSION, true );
		wp_localize_script(
			'dp-dfg-admin-cpt-modal',
			'dpdfg_admin',
			array(
				'ajaxurl'      => admin_url( 'admin-ajax.php' ),
				'nonce_cpt'    => wp_create_nonce( 'dpdfg_get_cpt_action' ),
				'nonce_tax'    => wp_create_nonce( 'dpdfg_get_taxonomies_action' ),
				'nonce_terms'  => wp_create_nonce( 'dpdfg_get_taxonomies_terms_action' ),
				'nonce_cf'     => wp_create_nonce( 'dpdfg_get_custom_fields_action' ),
				'nonce_ml_tax' => wp_create_nonce( 'dpdfg_get_multilevel_tax_action' ),
				'customFields' => self::get_custom_fields(),
				'i18n'         => array(
					'ml_tax_level_label' => __( 'Label text for', 'dpdfg-dp-divi-filtergrid' ),
					'ml_tax_level_all'   => __( 'All filter text for', 'dpdfg-dp-divi-filtergrid' ),
				)
			)
		);
	}

	/**
	 * Register our custom action to request the builder function on ajax.
	 *
	 * @since    1.0.0
	 */
	public function add_our_custom_action( $actions ) {
		return array_merge( $actions, array( 'dpdfg_get_posts_data_action' ) );
	}

}
